/*
 * Copyright (c) 2016-present Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0,
 * and you may not use this file except in compliance with the Apache License Version 2.0.
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Apache License Version 2.0 is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.nexus.ci.iq.PolicyEvaluationProjectAction

import org.sonatype.nexus.ci.iq.Messages
import org.sonatype.nexus.ci.iq.PolicyEvaluationHealthAction
import org.sonatype.nexus.ci.iq.PolicyEvaluationProjectAction

import jenkins.model.Jenkins
import groovy.json.JsonBuilder
import lib.JenkinsTagLib
import lib.LayoutTagLib


def t = namespace(JenkinsTagLib.class)
def l = namespace(LayoutTagLib.class)

def projectAction = (PolicyEvaluationProjectAction) it
def actions = projectAction.actions
def buildSummaries = projectAction.buildSummaries

if (actions && actions.size() > 0) {
  def policyCss = {
    // Inline the iq-chiclet css here for Jenkins v1 which does not support the css tag.
    style(type: 'text/css', """
        .iq-job-main-table {
          margin-top: 1em;
          margin-left: 1em;
        }

        .iq-block {
          display:inline-block;
          margin-right: 1em;
          float: left;
          line-height: 1.6;
          margin-bottom: 1.5em;
        }
        
        .iq-title {
          font-weight:bold;
          padding-right: 0.25em;
        }
        
        .iq-label {
          font-weight: bold;
        }
        
        .p-iq-chiclet {
          margin: 3px 0;
          padding-left: 0;
        }
        
        .iq-chiclet {
          display:inline-block;
          width:35px;
          text-align:center;
          border-radius:5px;
          -moz-border-radius:5px;
          color:white;
          margin-right: 3px;
          padding: 2px 1px;
          font-weight: bold;
        }
        
        .iq-chiclet.critical {
          background-color: #bc012f;
        }
        
        .iq-chiclet.severe {
          background-color: #f4861d;
        }
        
        .iq-chiclet.moderate {
          background-color: #f5c648;
        }
        
        .iq-chiclet-message {
          margin-left: 0.4em;
          padding-top: 2px;
        }     
      """)
  }

  for (PolicyEvaluationHealthAction action : actions) {
    def policyUI = {
      div(class: 'iq-block') {
        div(class: 'iq-title-container') {
          span(class: 'iq-title', Messages.IqPolicyEvaluation_LatestReportName())
          span(' - ')
          a(href: "${action.getUrlName()}", target: "_blank", " View Application Report")
          if (action.getPrioritiesLink()) {
            span(' | ')
            a(href: "${action.getPrioritiesLink()}", target: "_blank", "View Developer Priorities")
          }
        }
        if (action.getIqDisplayName()) {
          div() {
            span(class: 'iq-label', Messages.IqPolicyEvaluation_IqDisplayNameLabel())
            span(": ${action.getIqServerUrl()}")
            span(" (${action.getIqDisplayName()})")
          }
        }
        if (action.getApplicationId()) {
          div() {
            span(class: 'iq-label', Messages.IqPolicyEvaluation_ApplicationLabel())
            span(": ${action.getApplicationId()}")
          }
          div() {
            span(class: 'iq-label', Messages.IqPolicyEvaluation_StageLabel())
            span(": ${action.getIqStage()}")
          }
        }

        if (action.totalPolicyViolationCount == 0 && action.affectedComponentCount > 0) {
          div(class: 'p-iq-chiclet') {
            span(class: 'iq-chiclet critical', action.criticalComponentCount ? action.criticalComponentCount : 0)
            span(class: 'iq-chiclet severe', action.severeComponentCount ? action.severeComponentCount : 0)
            span(class: 'iq-chiclet moderate', action.moderateComponentCount ? action.moderateComponentCount : 0)
            span(' ')
            span(class: 'iq-chiclet-message',
                Messages.IqPolicyEvaluation_LegacyViolations(action.legacyPolicyViolationCount))
          }
        }
        else {
          div(class: 'p-iq-chiclet') {
            span(class: 'iq-label', Messages.IqPolicyEvaluation_TotalViolations(action.totalPolicyViolationCount))
            span(class: 'iq-chiclet-message',
                Messages.IqPolicyEvaluation_AffectedComponents(action.affectedComponentCount,
                    action.totalComponentCount ? action.totalComponentCount : 0))
          }
          div(class: 'p-iq-chiclet') {
            span(class: 'iq-chiclet critical',
                action.criticalPolicyViolationCount ? action.criticalPolicyViolationCount : 0)
            span(class: 'iq-chiclet severe', action.severePolicyViolationCount ? action.severePolicyViolationCount : 0)
            span(class: 'iq-chiclet moderate',
                action.moderatePolicyViolationCount ? action.moderatePolicyViolationCount : 0)
            span(' ')
            span(Messages.IqPolicyEvaluation_LegacyViolations(action.legacyPolicyViolationCount))
          }
        }
      }
    }

    table(class: 'iq-job-main-table') {
      t.summary(icon: '/plugin/nexus-jenkins-plugin/images/lifecycle-icon.svg', policyUI << policyCss)
    }
  }
}

if (buildSummaries && buildSummaries.size() > 0) {
  table(class: 'iq-job-main-table') {
    l.main_panel() {
      div(id: 'iqChart', style: 'margin-top: 1em')
      link(rel: "stylesheet",
          href: "${Jenkins.get().rootUrl}/plugin/nexus-jenkins-plugin/features/iq/charting/styles.css")
      script(src: "${Jenkins.get().rootUrl}/plugin/nexus-jenkins-plugin/lib/apexcharts.js")
      script(src: "${Jenkins.get().rootUrl}/plugin/nexus-jenkins-plugin/features/iq/charting/iqChart.js",
          chartTitle: Messages.IqPolicyEvaluation_ChartName(),
          buildSummaries: new JsonBuilder(buildSummaries).toString())
    }
  }
}
